//
// FS Classic 基板チェック用サンプルスケッチ
// Tokihiro FUKATSU @ NARO
// 2015/06/25 ver.1.01
//
// 電源投入後、LAN関係電源をONにしてWebサーバ起動
// Web上のボタンでLED、リレー、Twitterを制御可
//
#include <SPI.h>
#include <Ethernet.h>
#include <MsTimer2.h>
#include <Twitter.h>
#include <Time.h>
#include <EthernetUdp.h>

// Web,Twitter用雛型文章
PROGMEM prog_char version_name[] = "[FS-classic simple v1.01]";
PROGMEM prog_char web_header[] = "HTTP/1.0 200 OK\nContent-Type: text/html\nConnection: close\n";
PROGMEM prog_char web_top1[]    = "<html><head><title>Field Server Classic Web Server</title></";
PROGMEM prog_char web_top2[]    = "head><body><h3>Field Server Classic Simple ver.1.01</h3><hr/><form>";
PROGMEM prog_char web_top3[]    = "<table border>";
PROGMEM prog_char web_end[] = "</table></form><HR/>2013.9.4 by T.FUKATSU(NARO)</body></html>";

// インターネット接続関係
EthernetServer server(80);
byte mac[] = { 0x90, 0xA2, 0xDA, 0x01, 0x01, 0x01  }; //イーサネットシールドの裏を参照
byte  ip[] = { 192, 168, 0, 200 }; //メインコンピュータボードのIPアドレス
//byte dns_ip[] = { 192, 168, 0, 100 }; //dnsという名前だとエラー
//byte gateway[] = { 192, 168, 0, 100 }; //最後が*.1でない場合は設定必要
//byte subnet[] = { 255, 255, 255, 0 };

// Arduinoのpin設定
const byte DC5V_pin = 2;       //イーサシールド&無線用電源(CN10:5V/CN9:12V)をon/offするリレー
const byte DC12V_pin1 = 3;     //電動FANの電源(CN7:12V)をon/offするリレー
const byte DC12V_pin2 = 4;     //外部カメラの電源(CN8:12V)をon/offするリレー
const byte reset_pin = 5;      //イーサシールドのリセットピン(起動時にHにする必要有)
const byte LED_pin1 = 6;       //ステータス表示用LED
const byte LED_pin2 = 7;       //ステータス表示用LED
const byte MODE_pin = 8;       //外部入力(SW3が解放か短絡か)

// NTPサーバアクセス関係
const unsigned int localPort = 8888;
IPAddress timeServer(133, 243, 238, 164); //ntp.nict.jp
const byte NTP_PACKET_SIZE = 48;
byte packetBuffer[NTP_PACKET_SIZE];
EthernetUDP Udp;
byte hh = 0; //時刻同期したときのhourを保存
byte mm = 0; //時刻同期したときのminuteを保存
byte ss = 0; //時刻同期したときのsecondを保存

const unsigned long device_Wait = 500UL; //(ms) 動作安定のためのウェイト

//Twitterのトークン
int twitte_status = 0;
Twitter twitter("1968299359-7iVIEr7K48cnsybLOhNYAc6g5F0SesnBabil2Dx"); // fs-classic00

//データ取得時の変数
unsigned int raw[6]; // A/Dで得られた生電圧を保存(0-1023)
char str[64]; // コメント表示,Twitterなどテンポラリ文字列を格納
char msg[141]; // Twitteする本文を格納,最大140文字
String arg_data=""; //Webサーバアクセス時の引数を格納

// 制御用フラグ
boolean flag = true; //これがfalseになるとloop()は終了し、再度loop()を実行
boolean twitter_flag = false; //ツイートを行うかどうか
boolean mode_flag = false; //外部入力端子(SW3)が解放時はfalse、短絡時はtrue
boolean relay2_flag = false; //Web表示用
boolean relay1_flag = false; //Web表示用
boolean LED1_flag = false; //Web表示用
boolean LED2_flag = false; //Web表示用


//
// セットアップ
//
void setup(){ 

  // シリアル表示の設定
  Serial.begin(9600); 
  strcpy_P(str,(char*)version_name);
  Serial.println(str);

  // 入出力ピンの設定
  pinMode (DC5V_pin, OUTPUT);
  pinMode (DC12V_pin1, OUTPUT);
  pinMode (DC12V_pin2, OUTPUT);
  pinMode (reset_pin, OUTPUT);
  pinMode (LED_pin1, OUTPUT);
  pinMode (LED_pin2, OUTPUT);
  pinMode (MODE_pin, INPUT);

  // power on
  digitalWrite(reset_pin, LOW); //最初はLOWに設定
  digitalWrite(LED_pin1, HIGH); //最初はLEDを両方点灯
  LED1_flag = true;
  digitalWrite(LED_pin2, HIGH); //最初はLEDを両方点灯  
  LED2_flag = true;
  digitalWrite(DC5V_pin, HIGH); //イーサシールド&無線LAN+FANの電源ON
  digitalWrite(DC12V_pin1, LOW); //12V制御出力は最初off
  relay1_flag = false;
  digitalWrite(DC12V_pin2, LOW); //12V制御出力は最初off
  relay2_flag = false;
  delay(device_Wait); //不要かも？
  digitalWrite(reset_pin, HIGH); //Etherシールドをリセットして起動
//  analogReference(INTERNAL); //アナログ入力は0-1Vに切り替え

  //ネットワーク設定
  Ethernet.begin(mac, ip);
  // Ethernet.begin(mac, ip, dns_ip, gateway, subnet); //フルに記述する場合
  Udp.begin(localPort);
  delay(device_Wait);
  sendNTPpacket(timeServer); //AccessNTP内でも実行しているから不要かも？
  delay(device_Wait); //NTPサーバにうまくアクセスするため必要
 
  //NTP access
  AccessNTP();
  Udp.stop();
  digitalWrite(LED_pin1, LOW);
  LED1_flag = false;  
}


//
// メインループ
//
void loop(){
  
  // 初期設定
  flag = true;
  sprintf(msg,""); // 念のため初期化
  sprintf(str,""); // 念のため初期化
  
  if( twitter_flag ){
    // power on
    digitalWrite(reset_pin, LOW); //最初はLOWに設定
    digitalWrite(DC5V_pin, HIGH); //Etherシールド&無線LANの電源ON    
    delay(device_Wait); //不要かも？
    digitalWrite(reset_pin, HIGH); //Etherシールドをリセットしないとうまく動作しない
    Ethernet.begin(mac, ip);
    // Ethernet.begin(mac, ip, dns_ip, gateway, subnet); //フルに記述する場合
    //Twitteに書き込む内容を作成
    strcpy_P(msg, (char*)version_name);
    sprintf(str, "[%lu] from %d:%d:%d  ", millis()/1000, hh, mm, ss);
    strcat(msg,str);
    ReadSensor();
    sprintf(str ,"ad0:%d, ad1:%d, ad2:%d, ad3:%d, ad4:%d, ad5:%d",
                  raw[0], raw[1], raw[2], raw[3], raw[4], raw[5]); 
    strcat(msg, str);
    Serial.println(msg);
    ArduinoTwitte(msg);
    twitter_flag = false;
  }

  // Web server start
   Serial.println("web start");
   server.begin();
   while(flag){
      EthernetClient client = server.available();
      arg_data = ""; //Webサーバアクセス時の引数を初期化
      boolean arg_flag = true; //HTTPのヘッダを読み込み終えるまでtrue
      if (client && flag) {
        // an http request ends with a blank line
         boolean currentLineIsBlank = true;
         while (client.connected() && flag) {
            delay(1); //timing delay else webserver hangs.
            if (client.available() && flag) {
              char c = client.read();
              if( arg_flag ) arg_data.concat(c);
              if (c == '\n' && currentLineIsBlank) {
                //引数を確認して動作
                if(arg_data.indexOf("relay2=off") != -1 ){
                  digitalWrite(DC12V_pin2,LOW);
                  relay2_flag = false;
                }
                if(arg_data.indexOf("relay2=on") != -1 ){
                  digitalWrite(DC12V_pin2,HIGH);
                  relay2_flag = true;
                }              
                if(arg_data.indexOf("relay1=off") != -1 ){
                  digitalWrite(DC12V_pin1,LOW);
                  relay1_flag = false;
                }
                if(arg_data.indexOf("relay1=on") != -1 ){
                  digitalWrite(DC12V_pin1,HIGH);
                  relay1_flag = true;
                }
                if(arg_data.indexOf("LED1=off") != -1 ){
                  digitalWrite(LED_pin1, LOW);
                  LED1_flag = false;
                }
                if(arg_data.indexOf("LED1=on") != -1 ){
                  digitalWrite(LED_pin1, HIGH);
                  LED1_flag = true;
                }
                if(arg_data.indexOf("LED2=off") != -1 ){
                  digitalWrite(LED_pin2, LOW);
                  LED2_flag = false;
                }
                if(arg_data.indexOf("LED2=on") != -1 ){
                  digitalWrite(LED_pin2, HIGH);
                  LED2_flag = true;
                }                
                //ツイート(一度LAN周りをリブートしてから)
                if(arg_data.indexOf("tweet") != -1 ){
                  flag = false;
                  twitter_flag = true;
                }
   
                // Webレスポンス
                strcpy_P(str, (char*)web_header);
                client.println(str);                
                strcpy_P(str, (char*)web_top1);
                client.print(str);
                strcpy_P(str, (char*)web_top2);
                client.print(str);
                strcpy_P(str, (char*)web_top3);
                client.println(str);
                client.print(F("<tr><td>"));
                strcpy_P(msg, (char*)version_name);                  
                client.print(msg);
                client.print(F("</td></tr>"));
                client.print(F("<tr><td>"));                 
                unsigned long sec = millis()/1000;                
                sprintf(str, "[%lu sec]", sec);
                strcpy(msg,str);
                sprintf(str, " from %d:%d:%d", hh, mm, ss);
                strcat(msg,str);
                client.print(msg);
                client.println(F(" </td></tr>"));               
                ReadSensor();             
                for (int analogChannel = 0; analogChannel < 6; analogChannel++) {
                  client.print(F("<tr><td>ad"));
                  client.print(analogChannel);
                  client.print(F(":"));
                  client.print(raw[analogChannel]);
                  client.println(F("<br /></td></tr>"));
                }
               //SW3デジタル入力
                client.print(F("<tr><td>SW3:"));
                if( digitalRead(MODE_pin) == HIGH ){
                  client.print(F("HIGH"));
                } else {
                  client.print(F("LOW"));
                }
                client.println(F("</td></tr>"));
                client.print(F("<tr><td></td></tr>"));
                
                //操作ボタン
                client.print(F("<tr><td>relay1: <input type=\"submit\" name=\"relay1\" value=\"on\">"));
                if( relay1_flag ){
                  client.print(F("on"));
                } else {
                  client.print(F("off"));
                }
                client.print(F("<input type=\"submit\" name=\"relay1\" value=\"off\"></td></tr>"));

                client.print(F("<tr><td>relay2: <input type=\"submit\" name=\"relay2\" value=\"on\">"));
                if( relay2_flag ){
                  client.print(F("on"));
                } else {
                  client.print(F("off"));
                }
                client.print(F("<input type=\"submit\" name=\"relay2\" value=\"off\"></td></tr>"));

               client.print(F("<tr><td>LED1: <input type=\"submit\" name=\"LED1\" value=\"on\">"));
                if( LED1_flag ){
                  client.print(F("on"));
                } else {
                  client.print(F("off"));
                }
                client.print(F("<input type=\"submit\" name=\"LED1\" value=\"off\"></td></tr>"));

               client.print(F("<tr><td>LED2: <input type=\"submit\" name=\"LED2\" value=\"on\">"));
                if( LED2_flag ){
                  client.print(F("on"));
                } else {
                  client.print(F("off"));
                }
                client.print(F("<input type=\"submit\" name=\"LED2\" value=\"off\"></td></tr>"));

               //Twitterボタン
                client.print(F("<tr><td><p align=\"center\"><input type=\"submit\" name=\"tweet\" value=\"tweet\"></p></td></tr>"));
               //リロードボタン
                client.print(F("<tr><td><p align=\"center\"><input type=\"submit\" name=\"\" value=\"reload\"></p></td></tr>"));   
                
                // 残り                
                strcpy_P(str, (char*)web_end);
                client.println(str);
                Serial.println(F("http OK."));
                break;
              }
              if (c == '\n') {
                // you're starting a new line
                arg_flag = false;
                currentLineIsBlank = true;
              } 
              else if (c != '\r') {
                // you've gotten a character on the current line
                currentLineIsBlank = false;
              }
           }
         }
         // give the web browser time to receive the data
         delay(1);
         // close the connection:
         client.stop();    
      }
   }
   Serial.println(F("loop finish"));
    digitalWrite(DC5V_pin, LOW);
    digitalWrite(reset_pin, LOW);
    Udp.stop(); //念のため(不要?)   
}

//
// NTPリクエストを送信
//
unsigned long sendNTPpacket(IPAddress& address)
{
   // set all bytes in the buffer to 0
   memset(packetBuffer, 0, NTP_PACKET_SIZE); 
   // Initialize values needed to form NTP request
   // (see URL above for details on the packets)
   packetBuffer[0] = 0b11100011;   // LI, Version, Mode
   packetBuffer[1] = 0;     // Stratum, or type of clock
   packetBuffer[2] = 6;     // Polling Interval
   packetBuffer[3] = 0xEC;  // Peer Clock Precision
   // 8 bytes of zero for Root Delay & Root Dispersion
   packetBuffer[12]  = 49; 
   packetBuffer[13]  = 0x4E;
   packetBuffer[14]  = 49;
   packetBuffer[15]  = 52;
   // all NTP fields have been given values, now
   // you can send a packet requesting a timestamp:         
   Udp.beginPacket(address, 123); //NTP requests are to port 123
   Udp.write(packetBuffer,NTP_PACKET_SIZE);
   Udp.endPacket(); 
}

//
// NTPで時刻を取得
//
void AccessNTP(){
  Serial.println(F("send NTP packet"));
  sendNTPpacket(timeServer);
  delay(1000); //念のため(信州大のコードを参照)
  if ( Udp.parsePacket() ) {
     Udp.read(packetBuffer, NTP_PACKET_SIZE);
     // 時刻情報はパケットの40バイト目からはじまる4バイトのデータ
     unsigned long highWord = word(packetBuffer[40], packetBuffer[41]);
     unsigned long lowWord = word(packetBuffer[42], packetBuffer[43]);
     // NTPタイムスタンプは64ビットの符号無し固定小数点数
     //（整数部32ビット、小数部32ビット）
     // 1900年1月1日0時との相対的な差を秒単位で表している
     // 小数部は切り捨てて、秒を求めている
     unsigned long secsSince1900 = highWord << 16 | lowWord;
     // NTPタイムスタンプをUNIXタイムに変換する
     // UNITタイムは1970年1月1日0時からはじまる
     // 1900年から1970年の70年を秒で表すと2208988800秒になる
     const unsigned long seventyYears = 2208988800UL;
     // NTPタイムスタンプから70年分の秒を引くとUNIXタイムが得られる
     unsigned long epoch = secsSince1900 - seventyYears;
     // Timeライブラリに時間を設定(UNIXタイム)
     // 日本標準時にあわせるために＋9時間しておく
     setTime(epoch + (9 * 60 * 60));
     hh = hour();
     mm = minute();
     ss = second();
     sprintf(str, " %d : %d : %d", hh, mm, ss);
     Serial.println(str);
  } else {
    Serial.println(F("NTP failed."));
  } 
}

//
// A/D接続のセンサ値を読み込む(8回取りこんで平均する)
//
void ReadSensor(){
   for(byte analogChannel=0; analogChannel<6; analogChannel++) {
      analogRead(analogChannel); // dummy,チャンネル間の電圧吸い込み誤差を低減
      raw[analogChannel] = 0;
      for( byte i=0; i<8; i++){
         raw[analogChannel] += analogRead(analogChannel);
      }
      raw[analogChannel] = (raw[analogChannel]+4) >> 3; //切捨なので8/2を足してから8で割る
   }
}

//
// msgの内容をtwitteする
//
void ArduinoTwitte( char msg[141] )
{
  Serial.println(F("connecting ..."));
  if (twitter.post(msg)) {
    twitte_status = twitter.wait();
    if (twitte_status == 200) {
        Serial.println(F("twitte OK"));
    } else {
      Serial.print(F("failed : code "));
      Serial.println(twitte_status);
    }
  } else {
    twitte_status = -1;
    Serial.println(F("connection failed."));
  }
  Serial.println(F("twitte end"));
}

